/*
    include/comedi.h (installed as /usr/include/comedi.h)
    header file for comedi

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _COMEDI_H
#define _COMEDI_H

#ifdef __cplusplus
extern "C" {
#endif

#define COMEDI_VERSION "0.5.4"
#define COMEDI_VERSION_CODE (0*0x1000+5*0x100+4)

#define COMEDI_TRIGNOW 0
#define COMEDI_TRIGCLK 1
#define COMEDI_TRIGEXT 2
#define COMEDI_TRIG100HZ 3
#define COMEDI_TRIGSTOP 4

#define COMEDI_RDONLY 1
#define COMEDI_RDWR 0
#define COMEDI_SHARED 2


#define COMEDI_IO 0
 #define COMEDI_IO_INPUT 0
 #define COMEDI_IO_OUTPUT 1
 #define COMEDI_IO_BIDIR 2
#define COMEDI_AD 1
 #define COMEDI_AD_ANALOG 1
 #define COMEDI_AD_DIGITAL 0
#define COMEDI_BITS 2
#define COMEDI_MAXDATA 3
#define COMEDI_RANGE 4 /* integer describing a range selection -- user
	must look up actual range in a library */
#define COMEDI_LASTDATA 5  /* last data sample */
#define COMEDI_LASTSEC 6  /* time of last data sample (seconds) */
#define COMEDI_LASTNSEC 7  /* time of last data sample (nanoseconds) */
#define COMEDI_AREF 8  /* analog reference mode */
 #define COMEDI_AREF_GND 0  /* referenced to analog ground */
 #define COMEDI_AREF_COM 1  /* referenced to floating common */
 #define COMEDI_AREF_DIFF 2  /* referenced to another channel */
			     /* i.e., differential input */
 #define COMEDI_AREF_OTHER 3  /* some other reference */
#define COMEDI_DITHER 9  /* dithering, if available */
#define COMEDI_RANGE_TYPE 10


#define COMEDI_DEGLITCH 32  /* true=1 */

#define COMEDI_STROBE 48  /* strobed output true=1 */
#define COMEDI_IOBITGROUP 49  /* how many bits are grouped together for
				i/o designation */
#define COMEDI_IOBITS 50  /* bitfield describing i/o of each signal line */
 #define COMEDI_IOBITS_INPUT 0
 #define COMEDI_IOBITS_OUTPUT 0xffffffff

#define COMEDI_COUNT 64  /* count number for counters & reset */

typedef unsigned int sampl_t;

#define COMEDI_TRIG_INPUT	0	/* trigger input */
#define COMEDI_TRIG_OUTPUT	1	/* trigger input */
#define COMEDI_TRIG_BUF		2	/* use read/write to tranfer samples */

#define COMEDI_NDEVCONFOPTS 20
#define COMEDI_NAMELEN 20
#define COMEDI_NDEVICES 16
#define COMEDI_MAJOR 98
#define COMEDI_MAX_MCHAN 16

typedef struct{
	unsigned int chan;
	int io;			/* change this to config or something */
	unsigned int n;		/* change this to n_samples */
	int trigsrc;
	int trigvar;
	sampl_t data;
	unsigned int job;
}comedi_trig;

typedef struct{
	int io;
	unsigned int n1;	/* number of channels in scan */
	int trigsrc1;		/* time between samples */
	int trigvar1;
	unsigned int *chanlist;
	unsigned int n2;	/* number of scans */
	int trigsrc2;		/* time between scans */
	int trigvar2;
	unsigned int job;
}comedi_mtrig;

typedef struct{
	unsigned int job;
	unsigned int chan;
	sampl_t data;
}comedi_sample;

typedef struct{
	char name[COMEDI_NAMELEN];
	int options[COMEDI_NDEVCONFOPTS];
}comedi_devconfig;

typedef struct{
	unsigned int chan;
	unsigned int flags;
	unsigned int pnum;
	int pval;
}comedi_param;

typedef struct{
	char driver_name[COMEDI_NAMELEN];
	char board_name[COMEDI_NAMELEN];
	int chan;
	int iobase;
	int iosize;
	int irq;
	int minor;
	unsigned int version_code;
}comedi_info;

typedef struct{
	void *local;
	unsigned int remote;
	unsigned int nbytes;
}comedi_memcpy;

/* ioctls */

#define CIO 'd'
#define COMEDI_CONFIG _IOW(CIO,0,comedi_devconfig)
#define COMEDI_GETINFO _IOR(CIO,1,comedi_info)
#define COMEDI_TRIG _IOWR(CIO,2,comedi_trig)
#define COMEDI_MTRIG _IOWR(CIO,3,comedi_mtrig)

#define COMEDI_GETPARAM _IOWR(CIO,17,comedi_param)
#define COMEDI_SETPARAM _IOW(CIO,18,comedi_param)
#define COMEDI_GETNPARAM _IOWR(CIO,19,comedi_param)

#define COMEDI_COPY_FROM _IOW(CIO,32,comedi_memcpy)
#define COMEDI_COPY_TO _IOW(CIO,33,comedi_memcpy)
#define COMEDI_MEMSET _IOW(CIO,34,comedi_memcpy)
#define COMEDI_RESET _IO(CIO,35)
#define COMEDI_EXEC _IO(CIO,36)
#define COMEDI_LOCK _IO(CIO,37)
#define COMEDI_UNLOCK _IO(CIO,38)

#ifdef __cplusplus
}
#endif

#endif /* _COMEDI_H */

