/*
    include/comedilib.h
    header file for the comedi library routines

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _COMEDILIB_H
#define _COMEDILIB_H

#include <comedi.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct{
	double min;
	double max;
}comedi_range;

typedef struct{
	int n_ranges;
	comedi_range *range;
}comedi_range_info;

typedef struct{
	unsigned int maxdata;
	double offset;
	double max;
	double delta;
}comedi_voltage_info;

/*
   comedi_version_code is the version of the comedi source that
   the library came from.  It is used to verify compatibility with
   a running kernel module.
 */
extern unsigned int comedi_version_code;

int comedi_put(int fd,int chan,int data);
int comedi_masked_put(int fd,int chan,int mask,int data);
int comedi_get(int fd,int chan,int *data);
int comedi_getparam(int fd,int chan,int pnum,int *flags,int *pval);
int comedi_setparam(int fd,int chan,int pnum,int pval);
int comedi_getinfo(int fd,comedi_info *it);

int comedi_voltage_range(int fd,unsigned int chan,unsigned int range,
	comedi_voltage_info *vinfo);
int comedi_get_timer(int fd,unsigned int chan,double *f,
	unsigned int *trigvar);

int comedi_get_range_info(int fd,unsigned int chan,comedi_range_info **vinfo);


/*
extern inline int comedi_reset(int fd)
{
	return ioctl(fd,COMEDI_RESET,0);
}

extern inline int comedi_init(char *fn)
{
	return open(fn,O_RDWR);
}

extern inline int comedi_exec(int fd,int pc)
{
	return ioctl(fd,COMEDI_EXEC,pc);
}

int comedi_memcpy_to(int fd,void *src,int dest,int nbytes);
int comedi_memcpy_from(int fd,int src,void *dest,int nbytes);
int comedi_memset(int fd,int dest,int nbytes,int val);

int read_coff(FILE *it,int fd,int *entry_pc);
int load_and_run(int fd,char *fn);
*/


#ifdef __cplusplus
};
#endif

#endif

