/*
    module/atmio-16f.c
    hardware driver for National Instruments AT-MIO-16F

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1995 Claus Schroeter <clausi@chemie.fu-berlin.de>
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	This driver is an adaptation of one written by clausi
	
	specifications can be found in NI document ___
*/



/* Configuration and Status Group */

#define MIO_CMDR1       MIO_BASE+0x00
#define MIO_CMDR2       MIO_BASE+0x02
#define MIO_SR          MIO_BASE+0x00

/* Event Strobe Group */

#define MIO_SCONV       MIO_BASE+0x08
#define MIO_SDAQ        MIO_BASE+0x0a
#define MIO_ADCLR       MIO_BASE+0x0c
#define MIO_ESTR        MIO_BASE+0x0e
#define MIO_TCINT       MIO_BASE+0x16

/* Analog Output Group */

#define MIO_DAC0        MIO_BASE+0x10
#define MIO_DAC1        MIO_BASE+0x12
#define MIO_DUCLR       MIO_BASE+0x14
#define MIO_DUPD        MIO_BASE+0x0e

/* Analog Input Group */

#define MIO_MUXMEMLD    MIO_BASE+0x04
#define MIO_MUXMEMCLR   MIO_BASE+0x05
#define MIO_MUXMEM      MIO_BASE+0x06
#define MIO_ADFIFO      MIO_BASE+0x16

/* Counter/Timer Group */

#define MIO_CDATA       MIO_BASE+0x18
#define MIO_CCMD        MIO_BASE+0x1a
#define MIO_CSR         MIO_BASE+0x1a

/* Digital I/O */

#define MIO_DIR         MIO_BASE+0x1c
#define MIO_DOR         MIO_BASE+0x1c

/* RTSI Switch Group */

#define MIO_RSHIFT      MIO_BASE+0x1e
#define MIO_RSSTR       MIO_BASE+0x1f

/**********************************************************************
 *
 *  Commands thru CMDR1/2
 *
 *
 **********************************************************************/

/* commands for CMDR1 */

#define EEPROMCS (1<<15)
#define SDATA    (1<<14)
#define SCLK     (1<<13)
#define CALDACLD (1<<12)

#define DITHER   (1<<10)
#define INTGATE  (1<<9)
#define CAL      (1<<8)
#define DAQEN    (1<<7)
#define SCANEN   (1<<6)
#define SCN2     (1<<5)
#define SCANDIV  (1<<4)
#define CNT32    (1<<3)
#define AIG      (1<<2)
#define DIFF     (1<<1)
#define BIPOLAR  (1<<0)

/* commands for CMDR2 */

#define DIOPBEN  (1<<15)
#define DIOPAEN  (1<<14)
#define WGEN     (1<<13)
#define INTEN    (1<<12)
#define DMAEN    (1<<11)
#define DMATCINTEN (1<<10)
#define CMPLINTEN (1<<9)
#define DMAINTB2 (1<<8)
#define DMAINTB1 (1<<7)
#define DMAINTB0 (1<<6)
#define ADCFIFOREQ (1<<5)
#define RETRIG     (1<<4)
#define A4RCV      (1<<3)
#define A4DRV      (1<<2)
#define A2RCV      (1<<1)
#define A2DRV      (1<<0)

/* status register bits */

#define DAQCOMP (1<<15)
#define DAQPROG (1<<14)
#define FIFOHF  (1<<13)
#define FIFOEF  (1<<12)
#define DMATC   (1<<11)
#define DMATCA  (1<<10)
#define DMATCB  (1<<9)
#define OVERFLOW (1<<8)
#define OVERRUN  (1<<7)
#define DACUP    (1<<6)
#define DACUPERR (1<<5)
#define FACTCAL  (1<<4)
#define PROMOUT  (1<<3)
#define EEPROMCD (1<<2)
#define MUXMEMFF (1<<1)
#define MUXMEMEF (1<<0)

