/*
    module/comedi_module.h
    header file for kernel-only structures, variables, and constants

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _COMEDI_MODULE_H
#define _COMEDI_MODULE_H

#include <linux/version.h>

#if LINUX_VERSION_CODE < 2*65536+1*256+0
#define OLD_KERNEL 1
#else
#define OLD_KERNEL 0
#endif

#include <comedi.h>
#include <config.h>


typedef struct comedi_device_struct comedi_device;

typedef int comedi_devinit(comedi_device *,comedi_devconfig *);

typedef struct comedi_bufll{
	struct comedi_bufll *next;
	int use_count;
	char stuff[0];
}comedi_bufll;

typedef struct{
	comedi_bufll *first;
	comedi_bufll *last;
	int in;
	int use_count;
	int num_pages;
}comedi_buf;

typedef struct{
	comedi_buf *buf;
	comedi_bufll *bufll;
	int offset;
}comedi_bufptr;

typedef struct comedi_iparam{
	struct comedi_iparam *next;
	int flags;
	int pnum;
	int pval;
}comedi_iparam;

typedef struct{
	comedi_iparam *paramlist;
	int flags;
	int (*trig)(comedi_device *,comedi_trig *);
	int (*itrig)(comedi_device *,comedi_trig *);
	int (*sp)(comedi_device *,comedi_param *);
	comedi_iparam *lastdata,*lastsec,*lastnsec; /* shortcuts */
	void *lock;
}comedi_chinfo;

struct comedi_device_struct{
	int use_count;
	void *private;
	int minor;
	int iobase;
	int iosize;
	int irq;
	int chan;
	int polling;
	int busy[4];
	char *driver_name;
	char *board_name;
	comedi_chinfo *chinfo;
	comedi_buf rdbuf;
	struct wait_queue *rdwait;
	int (*rem)(comedi_device *);
	void (*poll)(comedi_device *);
	int (*write)(comedi_device *,const char *src,int nbytes);
	int (*memcpy_to)(comedi_device *,comedi_memcpy *);
	int (*memcpy_from)(comedi_device *,comedi_memcpy *);
	int (*d_memset)(comedi_device *,comedi_memcpy *);
	int (*memexec)(comedi_device *,int entry_pc);
	int (*reset)(comedi_device *,int entry_pc);
	int (*mtrig)(comedi_device *,comedi_mtrig *,unsigned int *);
};

extern comedi_device *comedi_devices;
extern comedi_devinit *comedi_devinits[];
extern const int comedi_maxinits;

/*
 * function prototypes
 */

int comedi_initbuf(comedi_buf *buf);
void comedi_freebuf(comedi_buf *buf);
void copy_to_buffer(comedi_buf *buf,void *src,int n);
int copy_from_buffer(comedi_bufptr *ptr,void *dest,int n);
comedi_bufptr *comedi_hookbuf(comedi_buf *buf);
void comedi_unhookbuf(comedi_bufptr *it);
void report_sample(comedi_device *dev,comedi_sample *s);
void comedi_error(comedi_device *dev,const char *s);

int init_param(void);
int cleanup_param(void);
int addparam(comedi_iparam **list,int pnum,int flags,int pval);
int changeparam(comedi_iparam **list,int pnum,int flags,int pval);
int delparamlist(comedi_iparam *list);
int getparamval(comedi_iparam *list,int pnum);

void init_polling(void);
void cleanup_polling(void);
void start_polling(comedi_device *);
void stop_polling(comedi_device *);

/* range types */
/* these *must* agree with lib/range.c */

#define RANGE_unipolar10		1
#define RANGE_unipolar5			2
#define RANGE_bipolar10			3
#define RANGE_bipolar5			4
#define RANGE_dt282x_ai_lo_bipolar	5
#define RANGE_dt282x_ai_lo_unipolar	6
#define RANGE_dt282x_ai_hi_bipolar	7
#define RANGE_dt282x_ai_hi_unipolar	8
#define RANGE_ni_E_ai			9
#define RANGE_unipolar2_5		10
#define RANGE_bipolar2_5		11
#define RANGE_dt282x_ai_5_bipolar	12
#define RANGE_dt282x_ai_5_unipolar	13
#define RANGE_pcl711b_ai		14
#define RANGE_ni_E_ai_limited		15


#endif /* _COMEDI_MODULE_H */

