/*
    module/ni_F.c
    hardware driver for National Instruments AT-MIO-16F

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1995 Claus Schroeter <clausi@chemie.fu-berlin.de>
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	This driver is an adaptation of one written by clausi
*/

#define Command_Register_1		0x00	/* wo */
#define Command_Register_2		0x02	/* wo */
#define Command_Register_3		0x04	/* wo */
#define Command_Register_4		0x06	/* wo */
#define Status_Register_1		0x18	/* ro */
#define Status_Register_2		0x1a	/* ro */

#define ADC_FIFO_Register		0x00	/* ro */
#define CONFIGMEM_Register		0x08	/* wo */

#define DAC0_Register			0x10	/* wo */
#define DAC1_Register			0x12	/* wo */

#define CONFIGMEMCLR_Register		0x1b	/* ro 8 */
#define CONFIGMEMLD_Register		0x1b	/* wo 8 */
#define DAQ_Clear_Register		0x19	/* ro 8 */
#define DAQ_Start_Register		0x1d	/* ro 8 */
#define Single_Conversion_Register	0x1d	/* wo 8 */
#define ADC_Calibration_Register	0x1f	/* wo 8 */

#define TMRREQ_Clear_Register		0x1f	/* ro 8 */
#define DAC_Update_Register		0x18	/* wo */
#define DAC_Clear_Register		0x1e	/* ro 8 */

#define DMA_Channel_Clear_Register	0x0b	/* ro 8 */
#define DMATCA_Clear_Register		0x19	/* wo 8 */
#define DMATCB_Clear_Register		0x09	/* ro 8 */
#define External_Strobe_Register	0x1e	/* wo 8 */
#define Calibration_DAC_0_Load_Register	0x0a	/* wo 8 */
#define Calibration_DAC_1_Load_Register	0x1a	/* wo 8 */

#define Am9513A_Data_Register		0x14	/* rw */
#define Am9513A_Command_Register	0x16	/* wo */
#define Am9513A_Status_Register		0x16	/* ro */

#define Digital_Input_Register		0x1c	/* ro */
#define Digital_Output_Register		0x1c	/* wo */

#define RTSI_Switch_Shift_Register	0x0c	/* wo 8 */
#define RTSI_Switch_Strobe_Register	0x0e	/* wo 8 */

#define _B(b)		((struct bitchan){_B_CHAN,(b)})

#define _B_CHAN		Command_Register_1
#define EEPROMCS	_bit15
#define SDATA		_bit14
#define SCLK		_bit13
#define SCANDIV		_bit12
#define INTGATE		_bit10
#define RETRIG_DIS	_bit9
#define DAQEN		_bit8
#define SCANEN		_bit7
#define SCN2		_bit6
#define CNT32		_bit5
#define RTSITRIG	_bit4
#undef _B_CHAN

#define _B_CHAN		Command_Register_2
#define A4RCV		_bit15
#define A4DRV		_bit14
#define A2RCV		_bit13
#define A2DRV		_bit12
#define BIPDAC1		_bit11
#define BIPDAC0		_bit10
#define EXTREFDAC1	_bit9
#define EXTREFDAC0	_bit8
#define EISA_DMA	_bit7
#define DMACHBB		_bits543
#define DMACHAB		_bits210
#undef _B_CHAN

#define _B_CHAN		Command_Register_3
#define 






#include <ni_F.h>

#define DB_ERROR      (1<<0)
#define DB_OVERFLOW   (1<<1)
#define DB_HALF       (1<<2)
#define DB_EMPTY      (1<<3)

#define DAQ_RUNNING (1<<0)
#define DAQ_DACUP   (1<<1)
#define DAQ_SCONV   (1<<2)
#define DAQ_BIPOLAR (1<<3)
#define DAQ_DIFF    (1<<4)
#define DAQ_SYNC    (1<<5)


typedef struct {
	int adc;
	int dac0;
} AdcVal;

typedef struct{
	int DaqMode;
	int CurrentDac;
	int WgenStat;
	int admode[16];
}ni_F_private;
#define devpriv ((ni_F_private *)dev->private)

void mio_SetDaqMode(int mode)
{
	DBGprint(DBG_DATA, ("Set DAQ-Mode (0x%x)", mode));
	devpriv->DaqMode = mode;
}

/**********************************************************************/
int mio_GetAdc(comedi_device * dev, comedi_trig * it)
{
	int stat;
	sampl_t buf;
	int adcval = 0;
	int dac;

	if (dev->busy[0])
		return -ETIME;

	outw(0x00, dev->iobase + MIO_ADCLR);

	/* set ADC UNIPOLAR/BIPOLAR mode */
	if (devpriv->admode[it->chan] & DAQ_BIPOLAR)
		mio_reset_bitcmd(BIPOLAR, 0);
	else
		mio_set_bitcmd(BIPOLAR, 0);

	/* set ADC DIFF/RSE mode */
	if (devpriv->admode[it->chan] & DAQ_DIFF)
		mio_set_bitcmd(DIFF, 0);
	else
		mio_reset_bitcmd(DIFF, 0);

	outw(0x00, dev->iobase + MIO_SCONV);

	/* poll status register */
	wait_for( inw(dev->iobase + MIO_SR) & FIFOEF,
		comedi_error(dev,"timeout");
		return -ETIME;
	}
/*
   while(!(inw(dev->iobaseMIO_SR) & FIFOEF ))
   dac=CurrentDac;
 */

	/* read value */
	it->data = inw(dev->iobase + MIO_ADFIFO);

	return 0;
}


#if 0
/***********************************************************************
 *
 *  Doublebuffer FIFO mechanism
 *
 */

#define MAX_DBUFSIZE 1024

struct wait_queue *DaqWait = NULL;
struct semaphore DaqSem = MUTEX;


unsigned int DblBuf[2][MAX_DBUFSIZE + 1];

unsigned int DblBufState = 0;

unsigned int ActualReadCount = 0;
unsigned int ActualWriteCount = 0;
unsigned int ActualResidue = 0;

unsigned int ActualWrite = 0;
unsigned int ActualRead = 0;

/***********************************************************************/
void mio_DaqInit(void)
{

	DblBufState = 0;
	ActualResidue = 0;
	ActualWriteCount = 0;
	ActualReadCount = 0;

	ActualRead = 0;
	ActualWrite = 0;
	DaqMode = 0;

	/*clear A/D circuit */
	outw(0x0000, MIO_ADCLR);

}


/**********************************************************************/
void mio_SwapWrite(void)
{
	ActualWrite = (ActualWrite ? 0 : 1);
	DBGprint(DBG_DATA, ("Swapped Write to %d", ActualWrite));
	ActualWriteCount = 0;
	if (ActualResidue >= (2 * MAX_DBUFSIZE)) {	/* overflow buffer bit set */
		DBGprint(DBG_DATA, ("Buffer Overflow"));
		DblBufState |= (DB_ERROR | DB_OVERFLOW);
	}
}

/**********************************************************************/
void mio_SwapRead(void)
{
	ActualRead = (ActualRead ? 0 : 1);
	DBGprint(DBG_DATA, ("Swapped Read to %d", ActualRead));
	ActualReadCount = 0;
}

/**********************************************************************/
void mio_AddDBuf(unsigned int val)
{
	if (ActualWriteCount < MAX_DBUFSIZE) {
		DblBuf[ActualWrite][ActualWriteCount] = val;
		ActualWriteCount++;
		ActualResidue++;
	} else {
		mio_SwapWrite();
	}
	if (ActualResidue >= MAX_DBUFSIZE) {
		DblBufState |= DB_HALF;
		if (DaqMode & DAQ_SYNC)
			wake_up_interruptible(&DaqWait);
	}
}

/**********************************************************************/
unsigned int mio_GetDBuf(void)
{
	unsigned int ret = 0;

	if (ActualResidue > 0) {
		if (ActualReadCount < MAX_DBUFSIZE) {
			ret = DblBuf[ActualRead][ActualReadCount];
			ActualReadCount++;
			ActualResidue--;
		} else {
			mio_SwapRead();
		}
		if (ActualResidue < MAX_DBUFSIZE)
			DblBufState &= ~DB_HALF;
	} else {
		ret = 0;
		DblBufState |= DB_EMPTY;
	}

	DBGprint(DBG_DATA, ("residue=%d", ActualResidue));
	return ret;
}

/**********************************************************************/
int mio_SyncRead(int *buf, int count)
{
	int retval, i;
	int dummy;
	char *tbuf;


	if (retval = verify_area(VERIFY_WRITE, buf, count))
		return retval;


	if (DblBufState & DB_EMPTY) {
		DBGprint(DBG_DATA, ("Buffer Empty"));
		return -EINTR;
	}
	if (!(DblBufState & DB_HALF) && (DaqMode & DAQ_RUNNING)) {
		DBGprint(DBG_DATA, ("\nWaiting for DB_HALF"));
		DaqMode |= DAQ_SYNC;
		interruptible_sleep_on(&DaqWait);
		DaqMode &= ~DAQ_SYNC;
		DBGprint(DBG_DATA, ("\nGot DB_HALF"));
	}
	if (DblBufState & DB_OVERFLOW) {
		DBGprint(DBG_DATA, ("Double Buffer Overflow"));
		return -EOVERFLOW;
	}
	/* copy block to FS */
	for (i = 0; i < count && i < ActualResidue; i++) {
		dummy = mio_GetDBuf();
		DBGprint(DBG_DATA, ("buf=%d", dummy));
		if (DblBufState & DB_EMPTY) {
			DBGprint(DBG_DATA, ("count=%d", i));
			return i;
		}
		put_fs_long(dummy, (int *) &(buf[i]));
		/*memcpy_tofs((int *) buf, &dummy ,sizeof(int)); */
	}

	return count;

}

/**********************************************************************/
void mio_DaqFinish(void)
{

	DaqMode &= ~DAQ_RUNNING;
	if (DaqMode & DAQ_SYNC)
		wake_up_interruptible(&DaqWait);
}

/**********************************************************************/
void mio_DaqStart(int mode)
{

	DBGprint(DBG_DATA, ("mode (0x%x)", mode));

	mio_DaqInit();

	DaqMode |= (DAQ_RUNNING | mode);

	/*clear A/D circuit */
	outw(0x0000, MIO_ADCLR);

	/* set ADC UNIPOLAR/BIPOLAR mode */
	if (DaqMode & DAQ_BIPOLAR)
		mio_reset_bitcmd(BIPOLAR, 0);
	else
		mio_set_bitcmd(BIPOLAR, 0);

	/* set ADC DIFF/RSE mode */
	if (DaqMode & DAQ_DIFF)
		mio_set_bitcmd(DIFF, 0);
	else
		mio_reset_bitcmd(DIFF, 0);

	if (DaqMode & DAQ_SCONV) {
		DBGprint(DBG_DATA, ("trigger conversion"));
		outw(0x00, MIO_SCONV);	/* trigger first value */
		while (!(inw(MIO_SR) & FIFOEF));	/* sync FIFOEF */
	}
	if (DaqMode & DAQ_DACUP)
		printk("WGEN MODE\n");

}

/***********************************************************************/
void mio_DaqIRQ(int status)
{
	comedi_sample s;

	if (DaqMode & DAQ_RUNNING) {

		/* read FIFO content into Buffer */
		s.chan = dev->adchan;
		s.job = dev->job;
		while (inw(MIO_SR) & FIFOEF) {

			it->data = inw(dev->iobase + MIO_ADFIFO);

		}

		/* trigger next conversion */
		if (DaqMode & DAQ_SCONV) {
			outw(0x0000, dev->iobase + MIO_SCONV);
		}
		if ((DaqMode & DAQ_DACUP) && !(WgenStat & WGEN_RUNNING)) {
#if DEBUG
			if (dbgMask)
				printk("Finish DACUP DAQ Cycle\n");
#endif
			DaqMode &= ~DAQ_RUNNING;
			while (!(inw(MIO_SR) & FIFOEF));	/* poll status */
			val = inw(MIO_ADFIFO) | (CurrentDac << 16);
			mio_AddDBuf(val);

			if (DaqMode & DAQ_SYNC)
				wake_up_interruptible(&DaqWait);
		}
	} else
		printk("DAQ-INTERRUPT unknown status (0x%x)", status);

}

#endif

int ni_F_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;
	
	if(strcmp(it->name,"ni_F")
		return 0;
	
	dev->driver_name="ni_F";
	dev->board_name="at-mio-16f";
	if(it->options[0])
		dev->iobase=it->options[0];
	else return -EINVAL;
	
	printk("comedi: ni_F: 0x%04x\n",dev->iobase);
	if(check_region(dev->iobase,NI_F_SIZE)<0){
		comedi_error(dev,"I/O port conflict");
		return -EIO;
	}
	request_region(dev->iobase,NI_F_SIZE,"ni_F");
	dev->iosize=NI_F_SIZE;
	
	dev->chan=16;
	
	dev->private=kmalloc(sizeof(ni_F_private)+
		dev->chan*sizeof(comedi_chinfo),GFP_KERNEL);
	if(!dev->private){
		return -ENOMEM;
	}
	memset(dev->private,0,sizeof(ni_F_private)+
		dev->chan*sizeof(comedi_chinfo));
	dev->chinfo=(void *)(devpriv+1);
	
	for(i=0;i<16;i++){
		dev->chinfo[i].itrig=ni_F_itrig;
		dev->chinfo[i].
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,12);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_MAXDATA,COMEDI_RDONLY,0xfff);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_ANALOG);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_INPUT);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_RANGE,0,0);
	}
	dev->rem=ni_F_rem;
	
	return 1;
}

static int ni_F_rem(comedi_device *dev)
{
	comedi_error(dev,"remove");
	
	kfree(dev->private);
	
	release_region(dev->iobase,dev->iosize);
	
	return 0;
}

