
#include <linux/kernel.h>
#include <linux/malloc.h>
#include <comedi_module.h>

#define N_PARAMS 1000

static comedi_iparam *paramarea,*freelist;

int init_param(void)
{
	int i;
	
	paramarea=kmalloc(N_PARAMS*sizeof(comedi_iparam),GFP_KERNEL);
	freelist=paramarea;
	if(!freelist)return -ENOMEM;

	memset(paramarea,0,N_PARAMS*sizeof(comedi_iparam));
	for(i=0;i<N_PARAMS-1;i++)
		freelist[i].next=freelist+i+1;
	freelist[N_PARAMS-1].next=NULL;
	
	return 0;
}

int cleanup_param(void)
{
	kfree(paramarea);

	return 0;
}

int addparam(comedi_iparam **list,int pnum,int flags,int pval)
{
	comedi_iparam *it;

if(!freelist)printk("comedi: parameter space too small\n");
	it=freelist;
	freelist=it->next;
	it->next=*list;
	it->flags=flags;
	it->pnum=pnum;
	it->pval=pval;
	*list=it;
	
	return 0;
}

int changeparam(comedi_iparam **list,int pnum,int flags,int pval)
{
	comedi_iparam *p=*list;
	
	while(p){
		if(p->pnum==pnum){
			p->pval=pval;
			p->flags=flags;
			return 0;
		}
		p=p->next;
	}
	addparam(list,pnum,flags,pval);
	return 0;
}

int delparamlist(comedi_iparam *it)
{
	comedi_iparam *next;

	while(it){
		next=it->next;
		it->next=freelist;
		freelist=it;
		it=next;
	}
	
	return 0;
}

int getparamval(comedi_iparam *it,int pnum)
{
	while(it){
		if(it->pnum==pnum)
			return it->pval;
		it=it->next;
	}
	return 0;
}


