/*
 * Driver for PCL725 and clones
 * David A. Schleef
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>



#define PCL725_SIZE 2

#define PCL725_DO 0
#define PCL725_DI 1

comedi_devinit pcl725_init;
static int pcl725_do(comedi_device *dev,comedi_trig *ad);
static int pcl725_di(comedi_device *dev,comedi_trig *ad);
static int pcl725_rem(comedi_device *dev);

static int pcl725_do(comedi_device *dev,comedi_trig *it)
{
	outb(dev->iobase+PCL725_DO,it->data);
	return 0;
}

static int pcl725_di(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	s.data=inb(dev->iobase+PCL725_DI);
	it->data=s.data;
	
	return 0;
}

int pcl725_init(comedi_device *dev,comedi_devconfig *it)
{
	if(strcmp("pcl725",it->name))
		return 0;

	dev->iobase=it->options[0];
	printk("comedi%d: pcl725: 0x%04x ",dev->minor,dev->iobase);
	if(check_region(dev->iobase,PCL725_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(dev->iobase,PCL725_SIZE,"pcl725");
	dev->driver_name="pcl725";
	dev->board_name="pcl725";
	dev->iobase=dev->iobase;
	dev->iosize=PCL725_SIZE;
	dev->irq=0;

	dev->chan=2;

	dev->chinfo=kmalloc(dev->chan*sizeof(comedi_chinfo),GFP_KERNEL);
	if(!dev->chinfo)return -ENOMEM;
	memset(dev->chinfo,0,dev->chan*sizeof(comedi_chinfo));

	dev->chinfo[0].itrig=pcl725_do;
	addparam(&dev->chinfo[0].paramlist,
		COMEDI_BITS,COMEDI_RDONLY,8);
	addparam(&dev->chinfo[0].paramlist,
		COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
	addparam(&dev->chinfo[0].paramlist,
		COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_INPUT);

	dev->chinfo[1].itrig=pcl725_di;
	addparam(&dev->chinfo[1].paramlist,
		COMEDI_BITS,COMEDI_RDONLY,8);
	addparam(&dev->chinfo[1].paramlist,
		COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
	addparam(&dev->chinfo[1].paramlist,
		COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_INPUT);

	dev->rem=pcl725_rem;
	printk("\n");

	return 1;
}


static int pcl725_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: pcl725: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(dev->chinfo);
	release_region(dev->iobase,dev->iosize);

	return 0;
}

